/*
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
 */
// vim: set tw=80 ts=2 sw=2 sts=2:

// The AccountController.
accountantApp

.factory("Accounts", ["$resource", function($resource) {
    return $resource(
        "/api/accounts/:id", {
            id: "@id"
        }
    );
}])

.controller(
    "AccountController", [
        "$scope", "$rootScope", "$routeParams", "Accounts",
        function($scope, $rootScope, $routeParams, Accounts) {

    $scope.accountClass = function(account) {
      if(account == $scope.selectedAccount) {
        return "active";
      }
    };

    $scope.valueClass = function(account, value) {
        if(!account) {
            return;
        }

        if(value < account.authorized_overdraft) {
            return "text-error";
        } else if(value < 0) {
            return "text-warning";
        }
    };

    /*
     * Create a new account and emit accountCreatedEvent.
     */
    $scope.createAccount = function(account) {
      account.$save(function(account) {
        // Reset new account.
        $scope.cancelEditAccount(account);

        $scope.$emit("accountCreatedEvent", account); 
      });
    };

    /*
     * Notify on account created event.
     */
    $rootScope.$on("accountCreatedEvent", function(e, account) {
      new PNnotify({
        type: "success",
        title: "Save",
        text: "Account #" + account.id + " created."
      });
    });

    /*
     * Reload accounts on account created event.
     */
    $rootScope.$on("accountCreatedEvent", function(e, account) {
      $scope.loadAccounts();
    });

    /*
     * Start account edition.
     */
    $scope.editAccount = function(account) {
      account.edit = true;
    };

    /*
     * Cancel account edition.
     */
    $scope.cancelEditAccount = function(account) {
      if(account.id) {
          // Reload account if it is a saved one.
          account.$get();
      } else {
          // Reset the new account.
          $scope.newAccount = new Accounts();
      }
    };

    /*
     * Save account and emit accountSavedEvent.
     */
    $scope.saveAccount = function(account, modalScope) {
      account.$save(function(data) {
        $scope.$emit("accountSavedEvent", account);
      });
    };

    /*
     * Notify on account saved event.
     */
    $rootScope.$on("accountSavedEvent", function(e, account) {
      new PNnotify({
        type: "success",
        title: "Save",
        text: "Account #" + account.id + " saved."
      });
    });

    /*
     * Delete an account and emut accountDeletedEvent.
     */
    $scope.deleteAccount = function(account, modalScope) {
      account.$delete(function(data) {
        $scope.$emit("accountDeletedEvent", account);
      });
    };

    /*
     * Notify on account deleted event.
     */
    $rootScope.$on("accountDeletedEvent", function(e, account) {
      new PNotify({
        type: "success",
        title: "Save",
        text: "Account #" + account.id + " deleted."
      });
    });

    /*
     * Reload acounts when an account is deleted.
     */
    $rootScope.$on("accountDeletedEvent", function(e, account) {
      // Reset the selected account if this is the one whe have deleted.
      if($scope.selectedAccount.id == account.id) {
        $scope.selectedAccount = null;
      }

      $scope.loadAccounts();
    });

    /*
     * Reload accounts and emit accountsLoadedEvent.
     */
    $scope.loadAccounts = function() {
      // Reset the new account.
      $scope.newAccount = new Accounts();

      // Load accounts using the resource.
      $scope.accounts = Accounts.query(function (data) {
        $scope.$emit("accountsLoadedEvent", $scope.accounts);
      });
    };

    /*
     * Update the selected account with loaded one or select the first of the
     * list.
     */
    $rootScope.$on("accountsLoadedEvent", function(e, accounts) {
      var accountToSelect = null;

      // Reset selected account to the new instance corresponding to the old
      // one.
      if($routeParams.accountId) {
        // Find the new instance of the previously selected account.
        angular.forEach($scope.accounts, function(account) {
          if(account.id == $routeParams.accountId) {
            // No need to emit accountSelectedEvent.
            $scope.selectedAccount = account;
          }
        });
      } else if($scope.selectedAccount) {
        // Find the new instance of the previously selected account.
        angular.forEach($scope.accounts, function(account) {
          if(account.id == $scope.selectedAccount.id) {
            // No need to emit accountSelectedEvent.
            $scope.selectedAccount = account;
          }
        });
      } else {
        // Set selected account to first one if not yet selected
        $scope.selectAccount($scope.accounts[0]);
      }
    });

    /*
     * Select the account to display and emit accountSelectedEvent.
     */
    $scope.selectAccount = function(account) {
      $scope.selectedAccount = account;

      $scope.$emit("accountSelectedEvent", {account: account});
    };

    /*
     * Refresh account information.
     */
    $scope.refreshAccount = function() {
        $scope.selectedAccount.$get();
    };

    $rootScope.$on("entrySavedEvent", $scope.refreshAccount);
    $rootScope.$on("entryCreatedEvent", $scope.refreshAccount);
    $rootScope.$on("entryRemovedEvent", $scope.refreshAccount);
    //$rootScope.$on("operationSavedEvent", $scope.refreshAccount);
    //$rootScope.$on("operationRemovedEvent", $scope.refreshAccount);

    $scope.loadAccounts();
}]);
